# frozen_string_literal: true

module Base32
  class Alphabet
    CHARS = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ234567'

    attr_reader :chars

    alias to_s chars

    def initialize(chars)
      self.chars = chars
    end

    def inspect
      "#<#{self.class}:#{chars.inspect}>"
    end

    def chars=(chars)
      unless chars_valid?(chars)
        raise ArgumentError, 'Chars must have 32 unique characters'
      end

      @chars = chars
    end

    def chars_valid?(chars)
      chars.bytes.to_a.size == 32 && chars.bytes.to_a.uniq.size == 32
    end
  end
end
